﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Slotnick_TextAdventure
{
    class GameObj
    {
        /// <summary>
        /// Monsters class
        /// </summary>
        public class Monster
        {
            string name;
            int totalHP;
            int currentHP;
            public Attack[] attacks;
            bool isAlive;
            bool attackOK;
            InventoryItem[] lootBag;
            string state;
            Talking[] response;

            public Monster(string tempName, int hp, Attack[] newAttack, Talking[] talk)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag = new InventoryItem[0];
                this.state = "Angry";
                this.response = talk;
            }

            public Monster(string tempName, int hp, Attack[] newAttack)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag = new InventoryItem[0];
                this.state = "Angry";
                List<Talking> chat = new List<Talking>();
                chat.Add(new Talking("Angry", "It has nothing to say.", "Angry"));
                this.response = chat.ToArray();
            }

            public Monster(string tempName, int hp, string status, Talking[] talk, bool atkOk)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = new Attack[0];
                this.isAlive = true;
                this.attackOK = atkOk;
                this.lootBag = new InventoryItem[0];
                this.state = "Angry";
                this.response = talk;
            }

            public Monster(string tempName, int hp, Attack[] newAttack, string monState)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag = new InventoryItem[0];
                this.state = monState;
                List<Talking> chat = new List<Talking>();
                chat.Add(new Talking(monState,"It has nothing to say.",monState));
                this.response = chat.ToArray();
            }

            public Monster(string tempName, int hp, Attack[] newAttack, string monState, GameObj.InventoryItem[] monLoot, Talking[] talk)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag =  monLoot;
                this.state = monState;
                this.response = talk;
            }
            public Monster(string tempName, int hp, Attack[] newAttack, string monState, GameObj.InventoryItem[] monLoot, Talking[] talk,bool attackable)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = attackable;
                this.lootBag = monLoot;
                this.state = monState;
                
                if(talk.Length==0)
                {
                    List<Talking> chat = new List<Talking>();
                    chat.Add(new Talking(monState,"It has nothing to say.",monState));
                    this.response = chat.ToArray();
                }
                else
                {
                    this.response = talk;
                }

            }

            public Monster(string tempName, int hp, Attack[] newAttack, string monState, GameObj.InventoryItem[] monLoot)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag = monLoot;
                this.state = monState;
                List<Talking> chat = new List<Talking>();
                chat.Add(new Talking(monState, "It has nothing to say.", monState));
                this.response = chat.ToArray();
            }

            public Monster(string tempName, int hp, Attack[] newAttack, string monState, Talking[] response)
            {
                this.name = tempName;
                this.totalHP = hp;
                this.currentHP = hp;
                this.attacks = newAttack;
                this.isAlive = true;
                this.attackOK = true;
                this.lootBag = new InventoryItem[0];
                this.state = monState;
                this.response = response;
            }

            public int TotalHP()
            {
                return this.totalHP;
            }

            public int CurrentHP()
            {
                return this.currentHP;
            }

            public bool IsAlive()
            {
                return this.isAlive;
            }
            public string MonsterName()
            {
                return this.name;
            }
            public bool Attackable()
            {
                return this.attackOK;
            }
            public string State()
            {
                return this.state;
            }
            public string Talk()
            {
                if (!this.IsAlive())
                {
                    return "It's dead, your not surprised that it can't speak.||Or are you?";
                }
                for (int x = 0; x < this.response.Length; x++)
                {
                    if (this.state == this.response[x].isState)
                    {
                        this.state = this.response[x].outState;
                        return this.response[x].dialog;
                    }
                }

                return "";
            }
            public void Kill()
            {
                this.isAlive = false;


                List<GameObj.InventoryItem> loot = Tools.worldMap[Tools.currentPosition.x, Tools.currentPosition.y].inventory.ToList<GameObj.InventoryItem>();
                foreach (GameObj.InventoryItem c in lootBag)
                {
                    loot.Add(c);
                }
                switch(Tools.RandNum(10))
                {
                    case 7:
                    case 8:
                        loot.Add(new GameObj.InventoryItem("Healing Potion", "When drunk heals you for 50hp", new GameObj.ItemEffect("hp", 50), "potion"));
                    break;
                    case 9:
                    case 10:
                    loot.Add(new GameObj.InventoryItem("Mana Potion", "When drunk heals you for 50mp", new GameObj.ItemEffect("mp", 50), "potion"));
                    break;
                    default:
                    break;
                }
                Tools.worldMap[Tools.currentPosition.x, Tools.currentPosition.y].inventory = loot.ToArray();
            }
            public void Damage(int damage)
            {
                this.currentHP -= damage;
                if (this.currentHP < 1)
                {
                    Kill();
                }
            }
            public bool Attack(int toHit)
            {
                int d100 = 100;
                if (Tools.RandNum(d100) >= toHit)
                {
                    return true;

                }

                return false;
            }

        }
        /// <summary>
        /// attack structure for monsters
        /// </summary>
        public struct Attack
        {
            public string name;
            public int hitChance;
            public int damage;

            public Attack(string tempName, int hit, int dam)
            {
                this.name = tempName;
                this.hitChance = hit;
                this.damage = dam;
            }
        }
        /// <summary>
        /// talking/interacting structure for npc/monsters
        /// </summary>
        public struct Talking
        {
            public string isState;
            public string dialog;
            public string outState;

            public Talking(string check, string speak, string become)
            {
                this.isState = check;
                this.dialog = speak;
                this.outState = become;
            }
        
        }
        /// <summary>
        /// room data struct
        /// </summary>
        public struct Room
        {
            public string shortDis;
            public string longDis;

            public Monster[] monsterList;
            public string enterEvent;
            public InventoryItem[] inventory;

            
            public Room(string shortD, string longD, Monster[] inMon, string enterEvent, InventoryItem[] inventory)
            {
                this.shortDis = shortD;
                this.longDis = longD;
                this.monsterList= inMon;
                this.enterEvent = enterEvent;
                this.inventory = inventory;
 
            }

            public bool HasMonsters()
            {
                if (this.monsterList.Length == 0)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            public bool HasItems()
            {
                if (this.inventory.Length == 0)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            /*
            public Room()
            {
                this.shortDis = "";
                this.longDis = "";
                this.monsterList = new Monster[5];

            }
            */
            

        }
        /// <summary>
        /// map data struct
        /// </summary>
        public struct Map
        {


        }
        /// <summary>
        /// item structure
        /// </summary>
        public struct InventoryItem
        {
            public string name;
            public string description;
            public ItemEffect effect;
            public string slot;
            public int count;

            public InventoryItem(string itemName, string itemDes, ItemEffect itemEffect, string slot)
            {
                this.name = itemName;
                this.description = itemDes;
                this.effect = itemEffect;
                this.slot = slot;
                this.count = 1;
            }
        }
        /// <summary>
        /// item effect structure
        /// </summary>
        public struct ItemEffect
        {
            public int toHit;
            public int damage;
            public int manaCost;
            public string stat;
            public int statBonus;
            
            public ItemEffect(int hit, int dmg) //weapon
            {
                this.toHit = hit;
                this.damage = dmg;
                this.stat = "";
                this.statBonus = 0;
                this.manaCost = 0;
                
            }
            public ItemEffect(int hit, int dmg, int mana) //spell
            {
                this.toHit = hit;
                this.damage = dmg;
                this.stat = "";
                this.statBonus = 0;
                this.manaCost = mana;

            }
            public ItemEffect(string statMod, int bonus) //potion
            {
                this.toHit = 0;
                this.damage = 0;
                this.stat = statMod;
                this.statBonus = bonus;
                this.manaCost = 0;
                
            }
            public ItemEffect(int toHitPenalty)  //armor
            {
                this.toHit = toHitPenalty;
                this.damage = 0;
                this.stat = "";
                this.statBonus = 0;
                this.manaCost = 0;
            }
            public ItemEffect(string words) //no effect  or quest
            {
                this.toHit = 0;
                this.damage = 0;
                this.stat = words;
                this.statBonus = 0;
                this.manaCost = 0;
            }

        }
    }
}
