﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Slotnick_TextAdventure
{
    class Tools
    {

        static public GameObj.Room[,] worldMap;
        static public PlayerPos currentPosition; 

        /// <summary>
        /// waits for a key to be pressed and returns they keypress
        /// </summary>
        /// <returns>Key pressed as a String</returns>
        public static string KeyPress()
        {
            //Console.Write("Press any key to continue...");

            System.ConsoleKeyInfo KInfo = Console.ReadKey(true);
            

            //Console.WriteLine("You have pressed :" + KInfo.Key.ToString());

            return KInfo.Key.ToString();
            
        }

        /// <summary>
        /// Gets a file and returns a bool if it was successful.
        /// If it is, then it takes the file, parses the rooms from it and returns a list containing the parsed room data as an out.
        /// </summary>
        /// <param name="filePath">Where to get the file you want to input and parse</param>
        /// <param name="output">the list you will out to if successful</param>
        /// <returns>bool of true or false depending on if the reader was successful</returns>
        public static string GetFromFile(string filePath)
        {

            StreamReader reader = null;
            try
            {
                reader = new StreamReader(filePath);
            }
            catch (Exception e)
            {
                Console.WriteLine("ERROR: File '" + filePath + "' could not be read.");
                Console.WriteLine("Problem: " + e.Message);
                Console.WriteLine("Please Try again.");

                //output = null;
                //return 0;
            }

            
            string inputFile = reader.ReadToEnd();
            reader.Close();
            return inputFile;
        }
        /// <summary>
        /// Parses a roomfile from an input file
        /// Format
        /// 
        /// Room
        /// Short:<short description>
        /// Long:<long description>
        /// Spawn:<spawn a npc actor/monster>
        /// Item:<spawn an item>
        /// </summary>
        /// <param name="inputFile">file to get room data</param>
        /// <returns></returns>
        public static GameObj.Room[,] RoomInput (string inputFile)
        {
       
            GameObj.Room[,] rooms = new GameObj.Room[150,150];
            
            string[] split = inputFile.Split(new Char[] { '\n' });


            for (int s = 0; s < split.Length; s++)
            {
                if (split[s].Trim() == "Room")
                {
                    
                    int x=0;
                    int y=0;
                    int outVal;
                    string tempLoc = "";
                    string tempShort = "";
                    string tempLong = "";
                    string tempSpawn = "";
                    string tempItem = "";
                    string tempEvent = "";
                    List<GameObj.Monster> roomMonster = new List<GameObj.Monster>();
                    List<GameObj.InventoryItem> items = new List<GameObj.InventoryItem>();

                    s++;
                    
                    for (; split[s].Trim() != ""; s++) //|| s < split.Length
                    {
                        if(split[s].Contains("Loc:"))
                        {
                            tempLoc = split[s].Remove(0, 4);
                            if (int.TryParse(tempLoc.Substring(0, tempLoc.IndexOf(',')).Trim(), out outVal))
                            {
                                x = outVal; 
                            }
                            if (int.TryParse(tempLoc.Substring(tempLoc.IndexOf(',') + 1).Trim(), out outVal))
                            {
                                y = outVal;
                            }
                        }
                        if (split[s].Contains("Short:"))
                        {
                            tempShort = split[s].Remove(0,6).Trim();
                           
                        }
                        if (split[s].Contains("Long:"))
                        {
                            tempLong = split[s].Remove(0, 5).Trim();
                            
                        }
                        if (split[s].Contains("Event:"))
                        {
                            tempEvent = split[s].Remove(0, 6).Trim();
                            
                        }
                        if (split[s].Contains("Item:"))
                        {
                            //add invintory parse
                            //items have 3 basic stats and 1 compount stat
                            //name, description, Effect(to hit+damage) slot
                            tempItem = split[s].Remove(0, 5).Trim();
                            string[] itemSplit = tempItem.Split(new Char[] { ',' });
                            GameObj.InventoryItem tempInvItem = new GameObj.InventoryItem();
                            List<GameObj.ItemEffect> tempEff = new List<GameObj.ItemEffect>();
                            string tempN = "";
                            string tempD = "";
                            string tempE = "";
                            
                            string tempSlot = "";
                            
                            foreach (string b in itemSplit)
                            {
                                if (b.Trim() != "")
                                {
                                    switch (b.Substring(0, 2))
                                    {
                                        case "N:":
                                            tempN = b.Trim().Remove(0, 2);
                                            break;
                                        case "D:":
                                            tempD = b.Trim().Remove(0, 2);
                                            break;
                                        case "E:":
                                            tempE = b.Trim().Remove(0, 2);
                                            break;
                                        case "T:":
                                            tempSlot = b.Remove(0, 2).Trim().ToLower();
                                            break;
                                        default:
                                            break;
                                    }
                                
                                }

                            }

                            int tempH = 0;
                            int tempDmg = 0;
                            int tempM = 0;
                            string tempStat = "";
                            int tempBonus = 0;
                            string[] eff = tempE.Trim().Split(new Char[] { ';' });

                            foreach (string t in eff)
                            {
                                if (t.Trim() != "")
                                {
                                    switch (t.Substring(0, 2))
                                    {
                                        case "H:":
                                            tempH = int.Parse(t.Remove(0, 2).Trim());
                                            break;
                                        case "D:":
                                            tempDmg = int.Parse(t.Remove(0, 2).Trim());
                                            break;
                                        case "M:":
                                            tempM = int.Parse(t.Remove(0, 2).Trim());
                                            break;
                                        case "S:":
                                            tempStat = t.Trim().Remove(0, 2);
                                            break;
                                        case "B:":
                                            tempBonus = int.Parse(t.Remove(0, 2).Trim());
                                            break;
                                        default:
                                            break;
                                    }
                                }
                            }

                            switch(tempSlot.Trim())
                            {
                                case "potion":
                                    tempInvItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempStat, tempBonus), tempSlot);
                                    break;
                                case "weapon":
                                    tempInvItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH, tempDmg), tempSlot);
                                    break;
                                case "armor":
                                    tempInvItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH), tempSlot);
                                    break;
                                case "spell":
                                    tempInvItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH, tempDmg,tempM), tempSlot);
                                    break;
                                case "quest":
                                    tempInvItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempStat), tempSlot);
                                    break;
                                default:
                                    break;
                            }

                            items.Add(tempInvItem);
                        }
                        if (split[s].Contains("Spawn:"))
                        {
                            tempSpawn = split[s].Remove(0, 6);
                            string[] monsterSplit = tempSpawn.Split(new Char[] { ',' });

                            List<GameObj.Attack> tempAttack = new List<GameObj.Attack>();

                            List<GameObj.InventoryItem> lootBag = new List<GameObj.InventoryItem>();
                            int hp = 0;
                            string tempName = "";
                            string tempS = "";
                            bool check = true;
                            List<string> tempTalk = new List<string>();
                            List<GameObj.Talking> speach = new List<GameObj.Talking>();
                            foreach (string b in monsterSplit)
                            {

                                if (b.Trim() != "")
                                {
                                    if (b.Contains("AtkOK:"))
                                    {
                                        if (b.Remove(0, 6).Trim().ToLower() == "no")
                                        {
                                            check = false;
                                        }
                                    }

                                    if (b.Contains("Talk:"))
                                    {
                                        tempTalk.Add(b.Remove(0, 5));
                                    }
                                    if (b.Contains("Name:"))
                                    {
                                        tempName = b.Remove(0, 5).Trim();
                                    }
                                    if (b.Contains("HP:"))
                                    {
                                        hp = int.Parse(b.Remove(0, 3).Trim());
                                    }
                                    if (b.Contains("State:"))
                                    {
                                        tempS = b.Remove(0, 6).Trim();
                                    }
                                    if (b.Contains("Attk:"))
                                    {
                                        //parse attack
                                        string[] attk = b.Remove(0, 5).Trim().Split(new Char[] { ';' });
                                        string tempN = "";
                                        int tempH = 0;
                                        int tempD = 0;
                                        string tempR = "";

                                        foreach (string c in attk)
                                        {
                                            switch (c.Substring(0, 2))
                                            {
                                                case "N:":
                                                    tempN = c.Trim().Remove(0, 2);
                                                    break;
                                                case "H:":
                                                    tempH = int.Parse(c.Trim().Remove(0, 2));
                                                    break;
                                                case "D:":
                                                    tempD = int.Parse(c.Trim().Remove(0, 2));
                                                    break;
                                                case "R:":
                                                    tempR = c.Remove(0, 2).Trim();
                                                    break;
                                                default:
                                                    break;
                                            }
                                        }
                                        tempAttack.Add(new GameObj.Attack(tempN, tempH, tempD));


                                    }

                                    if (b.Contains("Loot:"))
                                    {
                                        string tempLoot = b.Remove(0, 5).Trim();
                                        string[] itemSplit = tempLoot.Split(new Char[] { ';' });
                                        GameObj.InventoryItem tempLootItem = new GameObj.InventoryItem();
                                        List<GameObj.ItemEffect> tempEff = new List<GameObj.ItemEffect>();
                                        string tempN = "";
                                        string tempD = "";
                                        string tempE = "";

                                        string tempSlot = "";

                                        foreach (string d in itemSplit)
                                        {
                                            if (d.Trim() != "")
                                            {
                                                switch (d.Substring(0, 2))
                                                {
                                                    case "N:":
                                                        tempN = d.Trim().Remove(0, 2); //temp name
                                                        break;
                                                    case "D:":
                                                        tempD = d.Trim().Remove(0, 2); //temp long description
                                                        break;
                                                    case "E:":
                                                        tempE = d.Trim().Remove(0, 2); //temp effect (to be parsed below)
                                                        break;
                                                    case "T:":
                                                        tempSlot = d.Remove(0, 2).Trim().ToLower(); //temp slot
                                                        break;
                                                    default:
                                                        break;
                                                }

                                            }

                                        }
                                        int tempH = 0;
                                        int tempDmg = 0;
                                        int tempM = 0;
                                        string tempStat = "";
                                        int tempBonus = 0;
                                        string[] eff = tempE.Trim().Split(new Char[] { '#' });

                                        foreach (string t in eff)
                                        {
                                            if (t.Trim() != "")
                                            {
                                                switch (t.Substring(0, 2))
                                                {
                                                    case "H:":
                                                        tempH = int.Parse(t.Remove(0, 2).Trim());
                                                        break;
                                                    case "D:":
                                                        tempDmg = int.Parse(t.Remove(0, 2).Trim());
                                                        break;
                                                    case "M:":
                                                        tempM = int.Parse(t.Remove(0, 2).Trim());
                                                        break;
                                                    case "S:":
                                                        tempStat = t.Trim().Remove(0, 2);
                                                        break;
                                                    case "B:":
                                                        tempBonus = int.Parse(t.Remove(0, 2).Trim());
                                                        break;
                                                    default:
                                                        break;
                                                }
                                            }
                                        }

                                        switch (tempSlot.Trim())
                                        {
                                            case "potion":
                                                tempLootItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempStat, tempBonus), tempSlot);
                                                break;
                                            case "weapon":
                                                tempLootItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH, tempDmg), tempSlot);
                                                break;
                                            case "armor":
                                                tempLootItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH), tempSlot);
                                                break;
                                            case "spell":
                                                tempLootItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempH, tempDmg, tempM), tempSlot);
                                                break;
                                            case "quest":
                                                tempLootItem = new GameObj.InventoryItem(tempN, tempD, new GameObj.ItemEffect(tempStat), tempSlot);
                                                break;
                                            default:
                                                break;
                                        }
                                        lootBag.Add(tempLootItem);
                                    }

                                }
                            }
                            string tempIn = "";
                            string tempSay = "";
                            string tempOut = "";
                            foreach (string chat in tempTalk)
                            {
                                string[] chatSplit = chat.Split(new Char[] { ';' });
                                for (int i = 0; i < chatSplit.Length; i++)
                                {

                                    switch (chatSplit[i].Substring(0, 4).Trim())
                                    {
                                        case "InS:":
                                            tempIn = chatSplit[i].Remove(0, 4).Trim();
                                            break;
                                        case "Say:":
                                            tempSay = chatSplit[i].Remove(0, 4).Trim();
                                            break;
                                        case "Out:":
                                            tempOut = chatSplit[i].Remove(0, 4).Trim();
                                            break;
                                        default:
                                            break;
                                    }

                                    if (tempIn != "" && tempSay != "" && tempOut != "")
                                    {
                                        speach.Add(new GameObj.Talking(tempIn, tempSay, tempOut));
                                        tempIn = "";
                                        tempSay = "";
                                        tempOut = "";
                                    }

                                }
                            }

                            roomMonster.Add(new GameObj.Monster(tempName, hp, tempAttack.ToArray(), tempS, lootBag.ToArray(), speach.ToArray(), check));



                        }

                       
                    }
                    
                    rooms[x,y] = new GameObj.Room(tempShort,tempLong,roomMonster.ToArray(),tempEvent,items.ToArray());
                }
            }

            return rooms;

        }


        /// <summary>
        /// Holld the players current position
        /// </summary>
        public struct PlayerPos
        {
            public int x;
            public int y;

            public PlayerPos(int x, int y)
            {
                this.x = x;
                this.y = y;
            }
        }
        /// <summary>
        /// the menu
        /// </summary>
        /// <returns> string of what menu was chosen</returns>
        public static string PlayerMenuChoice()
        {
           string KP = KeyPress();

            switch (KP)
            {
                case "UpArrow":
                    View.cursorLocation[1]--;
                    if (View.cursorLocation[1] < 0)
                    {
                        View.cursorLocation[1] = 0;
                    }

                    break;
                case "DownArrow":
                    View.cursorLocation[1]++;
                    if (View.cursorLocation[1] >= View.basicMenu.Length && View.currentMenu == " ")//|| View.cursorLocation[1] >= View.selectedMenu.Length && View.currentMenu != " ")
                    {
                        View.cursorLocation[1]--;
                    }
                    break;
                case "RightArrow":
                case "Enter":
                    if (View.currentMenu == " ")
                    {
                        View.currentMenu = View.basicMenu[View.cursorLocation[1]].ToLower();
                        View.cursorLocation[0] = 10;
                        View.cursorLocation[1] = 0;
                    }
                    else
                    {
                        View.cursorLocation[0] += 10;
                    }
                   
                    break;
                case "LeftArrow":
                    View.cursorLocation[0]-=10;
                    if (View.cursorLocation[0] < 0)
                    {
                        View.cursorLocation[0]=0;
                    }
                    View.currentMenu = " ";
                    View.subMenu = " ";
                    break;
                case "Escape":
                    View.cursorLocation[0] = 0;
                    View.cursorLocation[1] = 0;
                    View.currentMenu = " ";
                    View.subMenu = " ";
                    return " ";
                default:
                    break;
            }
            View.tempMenu = null;
            if (worldMap[currentPosition.x, currentPosition.y].HasMonsters())
            {
                List<string> tempMenuContainer = new List<string>();
                for (int mon = 0; mon < worldMap[currentPosition.x, currentPosition.y].monsterList.Length; mon++)
                {
                    tempMenuContainer.Add(worldMap[currentPosition.x, currentPosition.y].monsterList[mon].MonsterName());
                }

                View.tempMenu = tempMenuContainer.ToArray();

            }
            View.itemMenu = null;
            if (worldMap[currentPosition.x, currentPosition.y].HasItems())
            {
                List<string> tempMenuContainer = new List<string>();
                for (int mon = 0; mon < worldMap[currentPosition.x, currentPosition.y].inventory.Length; mon++)
                {
                    tempMenuContainer.Add(worldMap[currentPosition.x, currentPosition.y].inventory[mon].name);
                }

                View.itemMenu = tempMenuContainer.ToArray();

            }
            View.backpack = null;
            if (Player.inventory.Length > 0)
            {
                List<string> tempMenuContainer = new List<string>();
                for (int mon = 0; mon < Player.inventory.Length; mon++)
                {
                    tempMenuContainer.Add(Player.inventory[mon].name);
                }
                
                View.backpack = tempMenuContainer.ToArray();
            }

            
            switch (View.currentMenu)
            {
                case "attack":
                    View.selectedMenu = View.tempMenu;

                    if (worldMap[currentPosition.x, currentPosition.y].HasMonsters() && View.cursorLocation[1] <= View.selectedMenu.Length - 1)
                    {
                        if (View.selectedMenu[View.cursorLocation[1]] == worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]].MonsterName() 
                            && View.cursorLocation[0] > 10 
                            && worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]].IsAlive()
                            )
                        {

                            View.thirdMenu = View.combatMenu;
                            View.subMenu = "combat";
                            if (worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]].Attackable())
                            {
                                PlayerMenuFight(worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]]);
                            }
                            View.cursorLocation[0] = 0;
                            View.cursorLocation[1] = 0;
                            View.currentMenu = " ";

                            View.subMenu = " ";

                        }
                        
                    }
                    else
                    {
                        View.cursorLocation[0] = 0;
                        View.cursorLocation[1] = 3;
                        View.currentMenu = " ";
                    }


                    break;
                case "interact":
                    View.selectedMenu = View.tempMenu;
                    if (worldMap[currentPosition.x, currentPosition.y].HasMonsters() && View.cursorLocation[1] <= View.selectedMenu.Length - 1)
                    {
                        if (View.selectedMenu[View.cursorLocation[1]] == worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]].MonsterName()
                            && View.cursorLocation[0] > 10
                            )
                        {
                            worldMap[currentPosition.x, currentPosition.y].enterEvent = worldMap[currentPosition.x, currentPosition.y].monsterList[View.cursorLocation[1]].Talk();
                            View.cursorLocation[0] = 0;
                            View.cursorLocation[1] = 0;
                            View.currentMenu = " ";
                        }
                        
                    }
                    else
                    {
                        View.cursorLocation[0] = 0;
                        View.cursorLocation[1] = 0;
                        View.currentMenu = " ";
                    }

                    break;
                case "use":
                    if (View.backpack.Length >= 2)
                    {
                        List<string> temp = new List<string>();
                        temp = View.backpack.ToList<string>();
                        temp.Remove("Empty");
                        View.backpack = temp.ToArray();
                    }
                    else if (View.backpack.Length == 1 && View.cursorLocation[0] > 10)
                    {
                        View.cursorLocation[0] = 0;
                        View.cursorLocation[1] = 2;
                        View.currentMenu = " ";
                    }


                        View.selectedMenu = View.backpack;

                        /*if (View.selectedMenu.Length > 9 && View.cursorLocation[1] >= 10)
                        {
                            List<string> newMenu = new List<string>();
                            newMenu = View.selectedMenu.ToList<string>();
                            newMenu.RemoveRange(0, 9);
                            View.selectedMenu = newMenu.ToArray();
                        }*/

                    if (Player.inventory.Length > 1)
                    {
                        if (View.cursorLocation[1] > View.selectedMenu.Length -1)
                        {
                            View.cursorLocation[1] = View.selectedMenu.Length - 1;

                        }
                        if (View.selectedMenu[View.cursorLocation[1]] == Player.inventory[View.cursorLocation[1] + 1].name && View.cursorLocation[0] > 10)
                        {

                            Player.UseItemFromInv(Player.inventory[View.cursorLocation[1] + 1]);
                            View.cursorLocation[0] = 0;
                            View.cursorLocation[1] = 0;
                            View.currentMenu = " ";

                        }
                        /*if (View.selectedMenu[View.cursorLocation[1]] == Player.inventory[View.cursorLocation[1] +10].name && View.cursorLocation[0] > 10)
                        {

                            Player.UseItemFromInv(Player.inventory[View.cursorLocation[1] + 10]);
                            View.cursorLocation[0] = 0;
                            View.cursorLocation[1] = 0;
                            View.currentMenu = " ";

                        }*/
                        
                    }
                    
                    
                    
                    break;
                case "take":
                    View.selectedMenu = View.itemMenu;
                    
                    if (worldMap[currentPosition.x, currentPosition.y].HasItems() && View.cursorLocation[1] <= View.selectedMenu.Length -1)
                    {
                        if (View.selectedMenu[View.cursorLocation[1]] == worldMap[currentPosition.x, currentPosition.y].inventory[View.cursorLocation[1]].name && View.cursorLocation[0] > 10)
                        {
                            Player.AddItemToInv(worldMap[currentPosition.x, currentPosition.y].inventory[View.cursorLocation[1]]);
                            List<GameObj.InventoryItem> temp = worldMap[currentPosition.x, currentPosition.y].inventory.ToList<GameObj.InventoryItem>();
                            temp.Remove(worldMap[currentPosition.x, currentPosition.y].inventory[View.cursorLocation[1]]);
                            worldMap[currentPosition.x, currentPosition.y].inventory = temp.ToArray();
                            View.cursorLocation[0] = 0;
                            View.cursorLocation[1] = 0;
                            View.currentMenu = " ";

                        }
                        
                    }
                    else
                    {
                        View.cursorLocation[0] = 0;
                        View.cursorLocation[1] = 1;
                        View.currentMenu = " ";
                    }
                    //View.selectedMenu = View.itemMenu;
                    

                    break;
                case "move":
                    View.currentMenu = "move";
                    View.selectedMenu = View.move;
                    View.tempMenu = null;
                    
                    Update();
                    while (PlayerMove())
                    {
                        Update();
                        if (worldMap[currentPosition.x, currentPosition.y].shortDis.ToLower() == "portal" )
                        {
                            return " ";
                        }
                        if (Player.HasThreeQuest() && worldMap[currentPosition.x, currentPosition.y].shortDis == "Broken Portal")
                        {
                            return " ";
                        }
                    }
                    View.cursorLocation[0] = 0;
                    View.cursorLocation[1] = 0;
                    View.currentMenu = " ";
                    Update();
                    break;
                case "inventory":
                 
                if(Player.inventory.Length >1)
                {
                    string toEvent = "";
                    for (int x = 1; x < Player.inventory.Length; x++)
                    {
                         toEvent += Player.inventory[x].name +"     x" + Player.inventory[x].count + "|     " + Player.inventory[x].description +"|"; 
                    }

                    if(Player.weapon.name != null)
                    {
                        toEvent+= "Weapon: "+Player.weapon.name + "|     " +  Player.weapon.description +"|";
                    }
                    if(Player.spell.name != null)
                    {
                        toEvent+= "Spell: "+Player.spell.name + "|     " +  Player.spell.description +"|";
                    }
                    if(Player.armor.name != null)
                    {
                        toEvent+= "Armor: "+Player.armor.name + "|     " +  Player.armor.description;
                    }
                    
                    worldMap[currentPosition.x, currentPosition.y].enterEvent = toEvent;
                }
                if (Player.inventory.Length == 1)
                {
                    string toEvent = "";
                    toEvent += Player.inventory[0].name + "|     " + Player.inventory[0].description + "|";

                    if (Player.weapon.name != null)
                    {
                        toEvent += "Weapon: " + Player.weapon.name + "|        " + Player.weapon.description + "|";
                    }
                    if (Player.spell.name != null)
                    {
                        toEvent += "Spell: " + Player.spell.name + "|       " + Player.spell.description + "|";
                    }
                    if (Player.armor.name != null)
                    {
                        toEvent += "Armor: " + Player.armor.name + "|       " + Player.armor.description;
                    }

                    worldMap[currentPosition.x, currentPosition.y].enterEvent = toEvent;
                }
                    View.cursorLocation[0] = 0;
                    View.cursorLocation[1] = 0;
                    View.currentMenu = " ";
                    Tools.KeyPress();
                break;
                default:
                    break;
            }
            /*
            if (View.selectedMenu == null)
            {
                View.currentMenu = " ";
                View.cursorLocation[1] = 0;
            }
            */
            

            return View.currentMenu;
        }
        /// <summary>
        /// the fight menu
        /// </summary>
        /// <param name="monster"> the moster your going to fight</param>
        public static void PlayerMenuFight(GameObj.Monster monster)
        {

            bool isFighting = true;
            
            while (isFighting)
            {
                string combatText = "You are fighting " + monster.MonsterName() + "\n\n";
                string KP = KeyPress();

                switch (KP)
                {
                    case "UpArrow":
                        View.cursorLocation[1]--;
                        if (View.cursorLocation[1] < 0)
                        {
                            View.cursorLocation[1] = 0;
                        }

                        break;
                    case "DownArrow":
                        View.cursorLocation[1]++;
                        if (View.cursorLocation[1] >= View.combatMenu.Length)//|| View.cursorLocation[1] >= View.selectedMenu.Length && View.currentMenu != " ")
                        {
                            View.cursorLocation[1]--;
                        }
                        break;
                    case "RightArrow":
                    case "Enter":

                        switch (View.combatMenu[View.cursorLocation[1]].ToLower())
                        {
                            case "melee":
                                if (Player.weapon.name != null)
                                {
                                    combatText += Attack(monster, Player.weapon.name, Player.weapon.effect.toHit, Player.weapon.effect.damage, Player.armor.effect.toHit);
                                }
                                break;
                            case "magic":
                                if (Player.spell.name != null)
                                {
                                    if (Player.spell.effect.manaCost < Player.currentMP)
                                    {
                                        Player.currentMP -= Player.spell.effect.manaCost;
                                        combatText += Attack(monster, Player.spell.name, Player.spell.effect.toHit, Player.spell.effect.damage, Player.armor.effect.toHit);
                                    }
                                    else
                                    {
                                        combatText += "Not enough mana";
                                    }
                                }
                                break;
                            case "flee":
                                if (worldMap[currentPosition.x, currentPosition.y + 1].shortDis != null)
                                {
                                    currentPosition.y++;
                                }
                                else if (worldMap[currentPosition.x + 1, currentPosition.y].shortDis != null)
                                {
                                    currentPosition.x++;
                                }
                                else if (worldMap[currentPosition.x - 1, currentPosition.y].shortDis != null)
                                {
                                    currentPosition.x--;
                                }
                                else if (worldMap[currentPosition.x, currentPosition.y-1].shortDis != null)
                                {
                                    currentPosition.y--;
                                }
                                View.cursorLocation[0] = 0;
                                View.cursorLocation[1] = 0;
                                View.currentMenu = " ";
                                isFighting = false;
                                break;
                            default:
                                break;
                        }
                        View.cursorLocation[1] = 0;
                        break;
                    case "LeftArrow":
                    case "Escape":


                        break;
                    default:
                        break;
                }

                combatText += "\n\n\n" + monster.MonsterName() + " has " + monster.CurrentHP() + "hp of " + monster.TotalHP() + "hp";
                
                if (!monster.IsAlive()||(currentPosition.x ==18&&currentPosition.y== 48))
                {
                    isFighting = false;
                    
                }
                else
                {
                    View.BuildFightDisplay(combatText);
                }

            }
        }
        /// <summary>
        /// moves the player around the map
        /// </summary>
        /// <returns></returns>
        public static bool PlayerMove()
        {
            bool done = false;
            while (!done)
            {
                string KP = KeyPress();
                switch (KP)
                {
                    case "UpArrow":
                        currentPosition.y--;
                        if (worldMap[currentPosition.x, currentPosition.y].shortDis == null)
                        {
                            currentPosition.y++;
                        }
                        return true;
                    case "DownArrow":
                        currentPosition.y++;
                        if (worldMap[currentPosition.x, currentPosition.y].shortDis == null)
                        {
                            currentPosition.y--;
                        }
                        return true;
                    case "RightArrow":
                        currentPosition.x++;
                        if (worldMap[currentPosition.x, currentPosition.y].shortDis == null)
                        {
                            currentPosition.x--;
                        }
                        return true;
                    case "LeftArrow":
                        currentPosition.x--;
                        if (worldMap[currentPosition.x, currentPosition.y].shortDis == null)
                        {
                            currentPosition.x++;
                        }
                        return true;
                    case "q":
                    case "Q":
                        done = true;
                        return false;
                    default:
                        return false;
                }
            }
            return true;
        }
        /// <summary>
        /// Builds and outputs the display to the screen
        /// </summary>
        private static void Update()
        {
            View.BuildDisplay();
            View.OutputDisplay();
        }
        /// <summary>
        /// Loads the real world for the begining of the game
        /// </summary>
        private static void PlayerStart()
        {
            worldMap = RoomInput(GetFromFile("RealWorld.txt"));
            currentPosition = new PlayerPos(10, 23);
            Update();
            while (PlayerMenuChoice() !="quit")
            {
                Update();
                if (worldMap[currentPosition.x, currentPosition.y].shortDis.ToLower() == "portal")
                {
                    currentPosition = new PlayerPos(18, 48);
                    KeyPress();
                    return;
                }

                
            }

        }
        /// <summary>
        /// load the real world for the end of the game
        /// </summary>
        private static void BackHome()
        {
            bool quit = false;
            worldMap = RoomInput(GetFromFile("RealWorld2.txt"));
            currentPosition = new PlayerPos(10, 23);
            Update();
            while (!quit)
            {
                PlayerMenuChoice();
                
                if(worldMap[currentPosition.x,currentPosition.y].enterEvent == "Thank you for playing")
                {
                    Update();
                    KeyPress();
                    break;
                   
                }
                 Update();
                 KeyPress();
            }

        }
        /// <summary>
        /// the start of the game
        /// </summary>
        public static void GameStart()
        {
            PlayerStart();
            worldMap = RoomInput(GetFromFile("OtherWorld.txt"));
            Update();
            while (PlayerMenuChoice()!="quit")// && worldMap[currentPosition.x,currentPosition.y].shortDis != "Broken Portal" )//PlayerMenuChoice()!="quit")
            {
                //PlayerMenuChoice();
                Update();
                if (Player.HasThreeQuest() && worldMap[currentPosition.x, currentPosition.y].shortDis == "Broken Portal")
                {
                    break;
                }
            }
            BackHome();

        }

        /// <summary>
        /// attacking function
        /// </summary>
        /// <param name="enemy">enemy monster</param>
        /// <param name="name">your attacks name</param>
        /// <param name="toHit">your to hit value</param>
        /// <param name="damage">how much damage you do</param>
        /// <param name="playerDefense">what % does your armor reduce your enemy's hit by</param>
        /// <returns></returns>
        public static string Attack(GameObj.Monster enemy, string name, int toHit, int damage, int playerDefense)
        {
            
            string combatText = "You attack " + enemy.MonsterName() + " with " + name + " and ";
            if (CheckHit(toHit,0) == "hit")
            {
                combatText = combatText +"hits dealing " + damage + ".";
                enemy.Damage(damage);
            }
            else
            {
                combatText += "miss.";
            }

            int pickAttack = RandNum(enemy.attacks.Length);
            if (pickAttack < 0 || pickAttack == enemy.attacks.Length)
            {
                pickAttack = 0;
            }

            combatText += "\n" + enemy.MonsterName() + " attacks you with " + enemy.attacks[pickAttack].name + " and "; 
            if (CheckHit(enemy.attacks[pickAttack].hitChance,playerDefense) == "hit")
            {
                combatText = combatText +"hits dealing " + enemy.attacks[pickAttack].damage + ".";
                Player.Damage(enemy.attacks[pickAttack].damage);
            }
            else
            {
                combatText += "misses.";
            }

            return combatText;
        }
        /// <summary>
        /// checks if something it
        /// </summary>
        /// <param name="toHit">total to hit</param>
        /// <param name="defense">total defense vs the hit</param>
        /// <returns></returns>
        public static string CheckHit(int toHit, int defense)
        {
            if(RandNum(100)< (toHit - defense))
            {
                return "hit";
            }
            return "miss";
        }


        /// <summary>
        /// makes random numbers from 0 to upTo
        /// </summary>
        static Random r = new Random();
        public static int RandNum(int upTo)
        {
            return r.Next(upTo);
        }
    }
}
